/* $Id: 20c505.h,v 1.2 1998/11/02 04:38:44 brian Exp $
******************************************************************************

   ATT20c505 / BT485 RAMDAC register definitions

   Copyright (C) 1995-1997		Steffen Seeger

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/


#ifndef _20c505_h
#define _20c505_h

#include "IBM/vga_dac.h"

/* extensions to Pixel Write Address Register */
	#define	DAC_PW_CRTL3_INDEX	0x01	/* CONTROL3 register index    */

#define	DAC_CW_INDEX	0x4	/* color write index register	*/

#define	DAC_C_DATA	0x5	/* color data (r/w) register	*/

#define	DAC_CONTROL0	0x6	/* DAC configuration register 0 */

	#define	DAC_C0_SLEEP_MODE	0x01	/* (CLK < 50MHz for wakeup!)  */
	#define	DAC_C0_8BIT		0x02	/* DAC's are 8 bits wide      */
	#define	DAC_C0_SYNC_ON_RED	0x04	/* composite sync on red      */
	#define DAC_C0_SYNC_ON_GREEN	0x08	/* composite sync on green    */
	#define	DAC_C0_SYNC_ON_BLUE	0x10	/* composite sync on blue     */
	#define	DAC_C0_BLANK_LEVEL	0x20	/* (only composite sync)      */
	#define	DAC_C0_NO_SLEEPSCLK	0x40	/* disable SCLK during sleep  */
	#define	DAC_C0_EXT_UNLOCK	0x80	/* unlock extended regs       */

#define	DAC_CR_INDEX	0x7	/* color read index register	*/

#define	DAC_CONTROL1	0x8	/* DAC configuration register 1 */

	#define	DAC_C1_HIGH_MUX		0x01	/* use high word (1?bit modes)*/
	#define	DAC_C1_SRC_CTRL		0x02	/* select pixel switch source */
	#define	DAC_C1_2_1_MUX		0x04	/* mux mode for 1?bit modes   */
	#define	DAC_C1_CLUT_BYPASS	0x10	/* bypass CLUT in 8 bit modes */
	#define DAC_C1_MODE_MASK	0xE8	/* these bits define the mode */

	#define	DAC_C1_24BPP		0x00	/* 24 bpp (888 weight) 	      */
	#define	DAC_C1_16BPP		0x28	/* 16 bpp (565 weight)	      */
	#define	DAC_C1_15BPP		0x20	/* 15 bpp (555 weight)        */
	#define	DAC_C1_8BPP		0x40	/*  8 bpp (indexed)	      */
	#define	DAC_C1_4BPP		0x60	/*  4 bpp (indexed)	      */
	#define	DAC_C1_1BPP		0x80	/*  1 bpp (indexed)           */


#define DAC_CONTROL2	0x9	/* DAC configuration register 2 */

	#define	DAC_C2_OFF_CURSOR	0x00	/* no hardware cursor	      */
	#define	DAC_C2_3CO_CURSOR	0x01	/* 3 color type cursor        */
	#define	DAC_C2_WIN_CURSOR	0x02	/* 2 color WIN type cursor    */
	#define	DAC_C2_X11_CURSOR	0x03	/* 2 color X11 type cursor    */
	#define	DAC_C2_CURSOR_TYPE	0x03	/* mask to extract cursor type*/
	#define	DAC_C2_PACKED_CLUT	0x04	/* packed or spaced CLUT      */
	#define	DAC_C2_INTERLACED	0x08	/* interlaced mode 	      */
	#define DAC_C2_PCLK0            0x00    /* select PCLK0               */
	#define	DAC_C2_PCLK1            0x10	/* select PCLK1               */
	#define	DAC_C2_ENHANCED_MODE	0x20	/* normal is VGA mode 	      */
	#define	DAC_C2_DAC_TEST		0x40	/* select test path	      */
	#define	DAC_C2_SCLK_DISABLE	0x80	/* disable SCLK output	      */

#define	DAC_STATUSREG	0xA	/* status register, read only   */

	#define	DAC_ST_MOD3_COUNTER	0x03	/* modulo 3 counter mask      */
	#define	DAC_ST_MPU_READ_CYCLE	0x04	/* last was read cycle        */
	#define DAC_ST_SENSE		0x08	/* sense state	              */

	#define	DAC_ST_ID_MASK		0xF0	/* ATT device type ID mask    */
		#define	DAC_BT485	0x80
		#define	DAC_ATT20C504	0xC0
		#define	DAC_ATT20C505	0xD0
		#define	DAC_ATT20C506	0xE0

#define	DAC_CURSOR	0xB	/* cursor pattern register	*/

#define DAC_CURSOR_XL	0xC	/* cursor location, low byte  */

#define	DAC_CURSOR_XH 	0xD	/* cursor location, high byte */

#define	DAC_CURSOR_YL	0xE	/* cursor location, low byte  */

#define	DAC_CURSOR_YH	0xF	/* cursor location, high byte */

/* CONTROL3 is an internal register and addressed indirectly. */

	#define	DAC_C3_ADDR_B8		0x01	/* ADDR bit 8		      */
	#define	DAC_C3_ADDR_B9		0x02	/* ADDR bit 9		      */
	#define	DAC_C3_ADDR		0x03	/* ADDR mask		      */
	#define	DAC_C3_LARGE_CURSOR	0x04	/* 64x64 instead of 32x32     */
	#define	DAC_C3_DOUBLE_CLK	0x08	/* double clock (505 only!)   */
	#define	DAC_C3_SWAP_NIBBLES	0x10	/* swap nibbles in 4bpp modes */
#endif /* _20c505_h */
