/* $Id: vga_io.inc,v 1.7 1998/12/06 06:31:53 brian Exp $
******************************************************************************

   IBM VGA Series/Standard -- I/O inline functions

   Copyright (C) 1995-1997		Jason McMullan	[jmcc@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

/*
 *	This code was derived from the following source of information:
 *	[1]	XFree86 vga driver
 */

/* Register definitions for IBM VGA are here: */
#include <IBM/vga.h>

/* Module intraface -- prototypes for functions in vga_io.c */
#include <kgi/kgi.h>

#define VGAIO_BASE   0x03C0
#define VGAIO_REGION 0x03E0-VGAIO_BASE

static kgiu16 INS1r;
static kgiu16 CRTI,CRTD;

static inline void outbMISC(kgiu8 value)
{
	outb(value, MISCw);			/* Put value      */
	if (value & MISC_COLOR_IO)		/* Adjust IO base */
	{ 
		
		CRTI = CRTIc; 
		CRTD = CRTDc; 
		INS1r = INS1rc; 
	}
	else
	{
		
		CRTI = CRTIm;
		CRTD = CRTDm;
		INS1r = INS1rm;
	};
}

static inline kgiu8 inbCRT(kgiu8 reg)
{ 
	outb(reg, CRTI);
	return inb(CRTD); 
}

static inline kgiu8 inbSEQ(kgiu8 reg)
{ 
	outb(reg, SEQI); 
	return inb(SEQD); 
}

static inline kgiu8 inbGRC(kgiu8 reg)
{ 
	outb(reg, GRCI); 
	return inb(GRCD); 
}

static inline kgiu8 inbATC(kgiu8 reg)
{ 
	outb(reg, ATCI); 
	return inb(ATCDr); 
}

static inline void outbCRT(kgiu8 value, kgiu8 reg) 
{ 
	outw((value << 8) | reg, CRTI); 
}

static inline void outbSEQ(kgiu8 value, kgiu8 reg)
{ 
	outw((value << 8) | reg, SEQI); 
}

static inline void outbGRC(kgiu8 value, kgiu8 reg)
{ 
	outw((value << 8 ) | reg, GRCI); 
}

static inline void outbATC(kgiu8 value, kgiu8 reg)
{ 
	outb(reg, ATCI); 
	outb(value, ATCDw); 
}

/* The macros below use a context macro to determine what state
   structure to work with.  This looks like so:

	#define WITH_STATE reset_state
	GRCBITS(GR06_MAP_A0_128, GR06_MAP_MASK, GRC_MMAP_MODE);
        #undef WITH_STATE

   Explicit versions of the macros are also defined.  The
   following does the same thing as the above, but is uglier
   when you are performaing a lot of operations on the same state:

	GRCBITS_ST(GR06_MAP_A0_128, GR06_MAP_MASK, 
	           GRC_MMAP_MODE, reset_state);
	
*/



/* Macro to alter bits in a stored state (which is then 
   flushed wholesale to the chipset.) */

/* OLD MACRO -- to be replaced */
#ifndef DISALLOW_DEPRICATED_CONSTRUCTS
# define KGI_POKEBITS(VAL,MASK,REG) \
	(REG) = (((REG) & (~MASK)) | ((MASK) & (VAL)))
#endif /* not DISALLOW_DEPRICATED_CONSTRUCTS */

/* NEW MACROS -- use these instead */
#define MISCBITS_ST(VAL, MASK, STATE) \
	(STATE).MISC = (((STATE).MISC & ~(MASK)) | ((MASK) & (VAL)))
#define MISCBITS(VAL, MASK) MISCBITS_ST((VAL), (MASK), WITH_STATE)

#define _BITS_ST(VAL, MASK, REGIDX, REGNUM, STATE) \
	(STATE).REGIDX[(REGNUM)] = \
	(((STATE).REGIDX[(REGNUM)] & ~(MASK)) | ((MASK) & (VAL)))

#define CRTBITS_ST(VAL, MASK, REGNUM, STATE) \
	_BITS_ST((VAL), (MASK), CRT, (REGNUM), (STATE))
#define CRTBITS(VAL, MASK, REGNUM) \
	CRTBITS_ST((VAL), (MASK), (REGNUM), (WITH_STATE))

#define SEQBITS_ST(VAL, MASK, REGNUM, STATE) \
	_BITS_ST((VAL), (MASK), SEQ, (REGNUM), (STATE))
#define SEQBITS(VAL, MASK, REGNUM) \
	SEQBITS_ST((VAL), (MASK), (REGNUM), (WITH_STATE))

#define GRCBITS_ST(VAL, MASK, REGNUM, STATE) \
	_BITS_ST((VAL), (MASK), GRC, (REGNUM), (STATE))
#define GRCBITS(VAL, MASK, REGNUM) \
	GRCBITS_ST((VAL), (MASK), (REGNUM), (WITH_STATE))

#define ATCBITS_ST(VAL, MASK, REGNUM, STATE) \
	_BITS_ST((VAL), (MASK), ATC, (REGNUM), (STATE))
#define ATCBITS(VAL, MASK, REGNUM) \
	ATCBITS_ST((VAL), (MASK), (REGNUM), (WITH_STATE))



/* Macros to alter bits in registers using a stored state to
   supply the values of unaltered bits.  */

#define ALTER_MISC_ST(VAL, MASK, STATE) \
	outbMISC(((STATE).MISC & ~(MASK)) | ((MASK) & (VAL)))
#define ALTER_MISC(VAL, MASK) ALTER_MISC_ST((VAL), (MASK), (WITH_STATE))

#define ALTER_CRT_ST(VAL, MASK, REGNUM, STATE) \
outbCRT(((STATE).CRT[(REGNUM)] & ~(MASK)) | ((MASK) & (VAL)), (REGNUM))
#define ALTER_CRT(VAL, MASK, REGNUM) \
ALTER_CRT_ST((VAL), (MASK), (REGNUM), (WITH_STATE))

#define ALTER_SEQ_ST(VAL, MASK, REGNUM, STATE) \
outbSEQ(((STATE).SEQ[(REGNUM)] & ~(MASK)) | ((MASK) & (VAL)), (REGNUM))
#define ALTER_SEQ(VAL, MASK, REGNUM) \
ALTER_SEQ_ST((VAL), (MASK), (REGNUM), (WITH_STATE))

#define ALTER_GRC_ST(VAL, MASK, REGNUM, STATE) \
outbGRC(((STATE).GRC[(REGNUM)] & ~(MASK)) | ((MASK) & (VAL)), (REGNUM))
#define ALTER_GRC(VAL, MASK, REGNUM) \
ALTER_GRC_ST((VAL), (MASK), (REGNUM), (WITH_STATE))

#define ALTER_ATC_ST(VAL, MASK, REGNUM, STATE) \
outbATC(((STATE).ATC[(REGNUM)] & ~(MASK)) | ((MASK) & (VAL)), (REGNUM))
#define ALTER_ATC(VAL, MASK, REGNUM) \
ALTER_ATC_ST((VAL), (MASK), (REGNUM), (WITH_STATE))

